package com.mh.user.job;

import com.mh.user.constants.Constant;
import com.mh.user.serialport.SerialPortSendReceive;
import com.mh.user.serialport.SerialPortThread;
import com.mh.user.service.DeviceCodeParamService;
import com.mh.user.service.DealDataService;
import com.mh.user.utils.AnalysisReceiveOrder485;
import com.mh.user.utils.GetReadOrder485;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author ljf
 * @title ：定时处理采集回来的历史数据
 * @description ：
 * @updateTime 2020-07-28
 * @throws ：
 */
@Slf4j
@Component
public class DealDataJob {

    private final DealDataService dealDataService;

    @Autowired
    private  DeviceCodeParamService deviceCodeParamService;

    private  GetReadOrder485 getReadOrder485;

    private static int taskTimes = 1;

    public DealDataJob(DealDataService dealDataService)
    {
        this.dealDataService = dealDataService;
    }

    /**
     * 定时处理汇总数据：每15分钟处理一次,十分钟(0 0/10 * * * ?)
     */
    @Scheduled(cron = "0 0/15 * * * ?")
    public void ProEnergy() {
        try {
            SimpleDateFormat sdf1=new SimpleDateFormat("yyyy-MM-dd HH:00:00");
            Date date=new Date();
            String curDate=sdf1.format(date);
            String name=dealDataService.customName();
            if (name!=null && name.length()>0 && name.contains("华夏学院")){
                dealDataService.proEnergy2(curDate);
            }else{
                dealDataService.proEnergy(curDate);    //yyyy-MM-dd HH:00:00
            }
            dealDataService.proGatewayState();         //判断网关在线状态：在线或离线
            log.info("进入定时调试数据库过程汇总数据！yyyy-MM-dd HH:00:00");
        } catch (Exception e) {
            e.printStackTrace();
            //Constant.FLAG=false;
            //Constant.WEB_FLAG=false;
        }
    }
    /**
     * 采集
     */
    @Scheduled(cron = "35 0/2 * * * ?")
//    @Scheduled(cron = "0/10 * * * * ?") 0 0/5 * * * ?
//    @Scheduled(cron = "0 0/5 * * * ?")  //5分钟
    public void collect() {
        try {
            log.info("------定时采集开始>>>>Constant.FLAG=="+Constant.FLAG+"------");
            SerialPortSendReceive sendReceive=new SerialPortSendReceive();
            if(Constant.FLAG==false){
                if(Constant.WEB_FLAG==false){
                    log.info("------taskTimes=="+taskTimes+"------");
                    if (taskTimes<=4) {
                        Constant.FLAG=true;
                        log.info("------Constant.WEB_FLAG=="+Constant.WEB_FLAG+"------");
                        if (taskTimes == 2) {//2
                            int r = deviceCodeParamService.queryCount2();  //查询记录数
                            if (r == 0) {
                              getReadOrder485.createOrderParam2();         //生成采集参数
                            }
                            for(int i=1;i<11;i++){
                                SerialPortThread myThread = new SerialPortThread();
                                Thread thread = new Thread(myThread);
                                myThread.setName("2", String.valueOf(i));
                                thread.start();
                            }
                            log.info("------采集水、电、运行状态！"+taskTimes+"------");
                        }else if (taskTimes == 3){//3
                            for(int i=1;i<11;i++){
                                SerialPortThread myThread = new SerialPortThread();
                                Thread thread = new Thread(myThread);
                                myThread.setName("1", String.valueOf(i));
                                thread.start();
                            }
                            log.info("------采集水位、水温！"+taskTimes+"------");
                        }else if (taskTimes == 4) {//4
                            for(int i=1;i<11;i++){
                                SerialPortThread myThread = new SerialPortThread();
                                Thread thread = new Thread(myThread);
                                myThread.setName("3", String.valueOf(i));
                                thread.start();
                            }
                            log.info("------采集设定温度、设定水位、故障状态！"+taskTimes+"------");
                        }else {
                            for(int i=1;i<11;i++){
                                SerialPortThread myThread = new SerialPortThread();
                                Thread thread = new Thread(myThread);
                                myThread.setName("1", String.valueOf(i));
                                thread.start();
                            }
                            log.info("------采集水位、水温！"+taskTimes+"------");
                        }
                        if(taskTimes<4){
                            taskTimes++;
                        }else{
                            taskTimes=1;
                        }
                    }
//                    Constant.FLAG=true;
//                    for(int i=1;i<4;i++){
//                        SerialPortThread myThread = new SerialPortThread();
//                        Thread thread = new Thread(myThread);
//                        myThread.setName("1",String.valueOf(i));
//                        thread.start();
//                        log.info("-------------采集水位、水温！线程"+String.valueOf(i)+"-------------");
//                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    /**
     * 定时处理数据：每十五分钟处理一次
     */
    @Scheduled(cron = "0 0/15 * * * ?")
    public void dealData() {
        try {
            SimpleDateFormat sdf1=new SimpleDateFormat("yyyy-MM-dd");
            Date date=new Date();
            String curDate=sdf1.format(date);
            String name=dealDataService.customName();
            if (name!=null && name.length()>0 && name.contains("华夏学院")){
                dealDataService.proEnergySum2(curDate);
            }else{
                dealDataService.proEnergySum(curDate);
            }
            dealDataService.proMaintainSum(curDate);   //汇总维修数
            dealDataService.proAlarmManage(curDate);   //报警信息
            dealDataService.proAlarmInfoSum(curDate);  //汇总相关警报数
            dealDataService.proAnalysisMonth(curDate);
            dealDataService.proAnalysisYear(curDate);
            dealDataService.proDeviceState(curDate);       //汇总设备状态
            dealDataService.proTotalPumpMinutes(curDate);  //统计周\月热泵运行时长
            log.info("进入定时调试数据库过程汇总数据！yyyy-MM-dd");
        } catch (Exception e) {
            e.printStackTrace();
            //Constant.FLAG=false;
            //Constant.WEB_FLAG=false;
        }
    }

    /**
     * 定时删除历史流水记录（删除前三个月的记录）
     */
//    @Scheduled(cron = "0 0 0 1 1/1 ? ")
//    public void deleteDataHistory() {
//        try {
//            dealDataService.deleteChillersDataHistory();
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }

}
